<?php
/** _    _          ___           __ _     (R)
 * | |  (_)_ _____ / __|___ _ _  / _(_)__ _
 * | |__| \ V / -_) (__/ _ \ ' \|  _| / _` |
 * |____|_|\_/\___|\___\___/_||_|_| |_\__, |
 *                                    |___/
 * $Id: lc-api.inc.php 1834 2012-09-19 19:39:59Z kk $
 * @author Keppler IT GmbH <info@liveconfig.com>
 * @copyright Copyright (c) 2009-2012 Keppler IT GmbH.
 * @version 1.1
 * --------------------------------------------------------------------------
 * Utility functions for LiveConfig IFRAME API
 * --------------------------------------------------------------------------
 */

/**
 * LiveConfig IFRAME API class
 *
 * <code>
 * require_once 'lc-api.inc.php';
 * $LC = new LC('https://liveconfig.example.org:8443/liveconfig/soap', 'admin', 'SoAp-PaSsWoRd');
 * $LC->checkSession('PaSsWoRd_fOr_tHiS_LiNk');
 * </code>
 *
 */
class LC {

  /**
   * SOAP client
   *
   * @var SoapClient
   */
  public $soap = NULL;

  /**
   * URL to the SOAP API
   *
   * @var string
   */
  private $soap_url;

  /**
   * LiveConfig login for SOAP API
   *
   * @var string
   */
  private $soap_login;

  /**
   * SOAP API password
   *
   * @var string
   */
  private $soap_pwd;

  /**
   * Last error message
   *
   * @var string
   */
  private $last_error;

  /**
   * User language (from session)
   *
   * @var string
   */
  private $user_language;

  /**
   * User login name (from session)
   *
   * @var string
   */
  private $user_login;

  /**
   * User name (from session)
   *
   * @var string
   */
  private $user_name;

  /**
   * Customer ID (from session)
   *
   * @var int
   */
  private $customer_id;

  /**
   * Create an instance of the LC class
   *
   * @param string $url URL to the LiveConfig SOAP API
   * @param string $login LiveConfig login
   * @param string $pwd SOAP API password
   */
  public function __construct($url, $login, $pwd) {
    $this->soap_url = $url;
    $this->soap_login = $login;
    $this->soap_pwd = $pwd;
  }

  /**
   * Create authentication token for SOAP requests
   *
   * @param string $fn method name
   * @param string $cust Customer ID to run method as
   * @return array SOAP authentication token
   */
  public function createToken($fn, $cust=NULL) {
    # Construct SOAP token:

    $ts = gmdate("Y-m-d") . "T" . gmdate("H:i:s") . ".000Z";
    if ($cust !== NULL) {
      $token = base64_encode(hash_hmac('sha1',
                                       'LiveConfig' . $this->soap_login . $fn . $ts . $cust,
                                       $this->soap_pwd,
                                       true
                                      )
                            );
    } else {
      $token = base64_encode(hash_hmac('sha1',
                                       'LiveConfig' . $this->soap_login . $fn . $ts,
                                       $this->soap_pwd,
                                       true
                                      )
                            );
    }
    $auth = array('login'     => $this->soap_login,
                  'timestamp' => $ts,
                  'token'     => $token,
                  'customer'  => $cust);
    return($auth);
  }

  /**
   * Return error message of recent function call
   *
   * @return string Error message
   */
  public function getLastError() {
    return($this->last_error);
  }

  /**
   * Return user language (from session)
   *
   * @return string Language code ("de", "en", ...)
   */
  public function getUserLanguage() {
    return($this->user_language);
  }

  /**
   * Return user login name (from session)
   *
   * @return string Login name
   */
  public function getUserlogin() {
    return($this->user_login);
  }

  /**
   * Return user name (from session)
   *
   * @return string User name (eg. "John Doe")
   */
  public function getUserName() {
    return($this->user_name);
  }

  /**
   * Return customer ID (from session)
   *
   * @return int Customer ID (eg. 12345)
   */
  public function getCustomerId() {
    return($this->customer_id);
  }

  /**
   * Check if session data is valid
   *
   * @param string $pwd IFRAME password
   * @return bool Return TRUE if session is valid
   */
  public function checkSession($pwd) {
    // check if all required variables are available:
    if (!isset($_REQUEST['ts']) || !isset($_REQUEST['sid']) || !isset($_REQUEST['chk'])) {
      $this->last_error = "missing request variable";
      return(false);
    }

    // generate SHA1 checksum on parameters
    $s = sha1($_REQUEST['ts'] . $_REQUEST['sid'] . $pwd);
    if ($_REQUEST['chk'] !== $s) {
      $this->last_error = "invalid checksum";
      return(false);
    }

    // Construct WSDL URL
    $wsdl_url = $this->soap_url
               .'?wsdl'
               .'&l=' . urlencode($this->soap_login)
               .'&p=' . urlencode($this->soap_pwd);

    try {
      if ($this->soap === NULL) {
        // Create SOAP client
        $this->soap = new SoapClient($wsdl_url,
                                     array('style'    => SOAP_DOCUMENT,
                                           'use'      => SOAP_LITERAL,
                                          )
                                    );
      }

      // now check user session:
      $params = array('auth'  => $this->createToken('SessionCheck'),
                      'sid'   => $_REQUEST['sid'],
                      'ua'    => $_SERVER["HTTP_USER_AGENT"],
                     );
      $response = $this->soap->SessionCheck($params);
      $this->user_language = $response->language;
      $this->user_login = $response->login;
      $this->user_name = $response->name;
      $this->customer_id = $response->cid;
      if (isset($response->subscriptions)) {
        $this->customer_subscriptions = $response->subscriptions;
        // adjust from "string" to "array" if only one subscription assigned:
        if (gettype($this->customer_subscriptions) == "string") $this->customer_subscriptions = array($this->customer_subscriptions);
      } else {
        // no subscriptions available - return empty array for convenience
        $this->customer_subscriptions = array();
      }

      // Debugging:
      /*
      print "<pre>";
      print_r($response);
      print "</pre>";
      */

    } catch (SoapFault $soapFault) {
      $err = $soapFault->faultstring;
      // replace eventual login/password from error message:
      $err = str_replace($this->soap_login, '###', $err);
      $err = str_replace(urlencode($this->soap_login), '###', $err);
      $err = str_replace($this->soap_pwd, '###', $err);
      $err = str_replace(urlencode($this->soap_pwd), '###', $err);
      $this->last_error=$err;
      return(false);
    }

    return(true);
  }

}

?>